unit genericU;

interface
uses SysUtils, Classes, Windows;

 type TGetCallback = function (PluginId: Integer; URL, Header: PChar): Integer; stdcall;
 type TCallType = (_Failure, _Start, _HTMLTag, _Stop, _Unparsed);

 procedure KSGetPluginDescription(var Buf; BufLen: Integer); stdcall; export;
 procedure KSInitPlugin(GetCallback: TGetCallback; PluginId: Integer; RootDir: PChar); stdcall; export;
 procedure KSStartJob(ABaseURL: PChar); stdcall; export;
 procedure KSParseDoc(Id, Calltype: Integer; Param1, Param2, Param3: Pointer; Len: Integer); stdcall; export;
 procedure KSDonePlugin; stdcall; export;
 procedure KS1Min; stdcall; export;

 var Lock: TRTLCriticalSection;
     StartURL: TGetCallback;
     _PluginId: Integer;

implementation

//

 procedure SplitBy(Src,SubStr: String; var Dest1,Dest2: String);
 var APos: Integer;
     Temp: String;
 begin

  APos := Pos(AnsiUpperCase(Substr),AnsiUpperCase(Src));
  if APos = 0 then
  begin
   Dest1 := Src;
   Dest2 := '';
   Exit;
  end;

  Temp := Src;
  Dest1 := Copy(Temp,1,APos-1);
  Dest2 := Copy(Temp,APos + Length(SubStr), Length(Temp) - APos - Length(SubStr) + 1);

 end;

//

 function GetParamValue(TagBody, ParamName: String): String;
 var L, R: String;
     Ind: Integer;
 begin

  Result := '';
  if (TagBody = '') or (ParamName = '') then Exit;

  SplitBy(TagBody, ParamName + '=', L, R);
  if R <> '' then
  begin

   R := Trim(R);

   if R <> '' then
   begin

    if R[1] = '"' then
     for Ind := 2 to Length(R) do
      if (R[Ind] < #32) or (R[Ind] = '"') then Break else Result := Result + R[Ind]
    else
     for Ind := 1 to Length(R) do
      if R[Ind] <= #32 then Break else Result := Result + R[Ind];

   end;

  end;

 end;

//

 procedure KSGetPluginDescription(var Buf; BufLen: Integer);
 begin
  StrLCopy(PChar(@Buf), 'Sample kindaspider plugin, kindastub.', BufLen);
 end;

//

 procedure KSInitPlugin(GetCallback: TGetCallback; PluginId: Integer; RootDir: PChar);
 begin

  // GetCallback points to the internal KS callback of a type TGetCallback
  // PluginId is an unique Id for this plugin, which is to be passed
  // back to the GetCallback for KS to know to which plugin the call
  // belongs

  // type TGetCallback = function (PluginId: Integer; URL, Header: PChar): Integer; stdcall;

  // GetCallback tells KS to start loading of a document "URL", i.e. "http://asite/doc.htm"
  // The default HTTP request will look like:
  // -
  // GET URL HTTP/1.0
  // User-Agent: KindaSpider/1
  // -

  // if you wish to add for instance "Cookie: JHGJGD" to the header,
  // just pass "Cookie: JHGJGD" in a Header parameter

  // For debug needs you may pass -1 instead of the PluginId,
  // in that case URL PChar is being added to the log, but not
  // being started

  StartURL := GetCallback;
  _PluginId := PluginId;

 end;

//

 procedure KSDonePlugin;
 begin
  // do the cleanup
 end;

//

 procedure KSStartJob(ABaseURL: PChar); stdcall; export;
 begin
  // start retrieving from the passed URL
  if Assigned(StartURL) then StartURL(_PluginId, ABaseURL, nil);
 end;

//

 procedure KSParseDoc(Id, Calltype: Integer; Param1, Param2, Param3: Pointer; Len: Integer);
 begin

  // this proc may be called from within arbitrary number of threads simultaneously
  // Id is an unique Id for a HTTP retrieved document
  // Calltype is of a type TCalltype, see below
  // Param1..3 mean something for each very Calltype
  // Len is not actually length, it's just called so

  EnterCriticalSection(Lock);           // make it thread safe
  try

   case TCallType(Calltype) of
   _Start:                     // document just has been loaded, and
   begin                       // for this is a HTML doc, parsing was started

    // Param1 is a full URL of a loaded document, i.e. "http://asite.com/doc.htm"
    // Param2 is a PChar, pointing to the top of the entire HTTP document
    // Param3 is a PChar, pointing to the top of the content
    // Len is a HTTP retcode, 200, 300, 404 etc.

    StartURL(-1, PChar('_Start: ' + PChar(Param1)), nil);

   end;
   _Stop:                      // document parsing has been ended, no more calls
   begin                       // with this Id will be issued

    // Params are nil, Len is 0

    StartURL(-1, PChar('_Stop: ' + PChar(Param1)), nil);

   end;
   _Failure:                   // loading of a document failed
   begin

    // Params are nil, Len is 0

   end;
   _HTMLTag:                   // after _start, parser calls this for each
   begin                       // tag it walks through

    if Param1 <> nil then
     StartURL(-1, PChar('_Tag: ' + PChar(Param1)), nil);

    // Param1 is a tag name in the uppercase
    // Param2 is a tag body
    // Param3 is a plain text fragment between this tag and the
    // previous one
    // Len is 0

    // Example: assuming, that the A tag is being parsed:
    // <B>click <A HREF="url">here</A></B>
    //    ^^^^^  ^ ^^^^^^^^^^
    //    3      1 2
    // so, param1 is "A", param2 is "HREF="url""
    // param3 is "click "

    // or, if some tet is found after the last tag,
    // Param1 is nil,
    // Param2 is nil,
    // Param3 is this text "tail"

    // Example:
    // </HTML>Some more text ?
    //        ^^^^^^^^^^^^^^^^
    //        3

   end;
   _Unparsed:                  // document was loaded, but it's not a HTML one
   begin                       // so, parsing is skipped, no _start is called

    StartURL(-1, PChar('_Unparsed: ' + PChar(Param1)), nil);

    // Param1 is a full URL of a loaded document
    // Param2 is a PChar, pointing to the top of the entire HTTP document
    // Param3 is nil
    // Len is a length of the document

   end;
   end;

  finally
   LeaveCriticalSection(Lock);
  end;

 end;

//

procedure KS1Min; stdcall;
begin

 // this proc is called once a minute and will not will be called
 // again until the previous call has been returned

end;

//

initialization
 InitializeCriticalSection(Lock);
finalization
 DeleteCriticalSection(Lock);
end.
